/**
 * @author till.reitemeyer + andy.gutsche + sebastian.fitzner
 */
(function($) {
    $.widget('carousel.slideshow', {
        options: {
            nextLink: '.pagination__next',
            prevLink: '.pagination__prev',
            slides: '.list',
            slide: '.list__item',
            caption: '.caption',
            autoScroll: false, // true, false as option
            speed: 2500,
            setParentWidth: false
        },

        _create: function() {
            this.loadedSlideImgs = 0;
            this.$slideImgs = $('img', this.element);
            this.totalSlideImgs = this.$slideImgs.length || 0;

            this._checkSlideLoadProgress();
        },

        _checkSlideLoadProgress: function() {
            var timeout;
            var that = this;

            this.loadedSlideImgs = 0;

            this.$slideImgs.each(function() {
                if (this.complete) {
                    that.loadedSlideImgs++;
                }
            });

            if (this.loadedSlideImgs === this.totalSlideImgs) {
                clearTimeout(timeout);

                timeout = setTimeout(function(e) {
                    that._initSlideshow();
                }, 0);
            } else {
                timeout = setTimeout(function(e) {
                    that._checkSlideLoadProgress();
                }, 250);
            }
        },

        _initSlideshow: function() {
            var timeout;
            var resizeTimer;
            var that = this;

            this.$prev = this.element.find(this.options.prevLink);
            this.$next = this.element.find(this.options.nextLink);
            this.slides = this.element.find(this.options.slide);
            this.$caption = this.element.find(this.options.caption);
            this.index = 0;

            this.slides.addClass('active');
            // this.$caption.text(this.slides[0].getAttribute('data-caption'));

            this.$prev.on('click', function(e) {
                that.gotoSlide(that.index - 1);
            });

            this.$next.on('click', function(e) {
                that.gotoSlide(that.index + 1);
            });

            // Enabling autoscroll
            if (this.options.autoScroll === true || this.element.attr('data-carousel-autoscroll') == "true") {
                this.enableAutoScroll();
            }

            if (window.Modernizr && Modernizr.touch) {
                this.enableTouchSupport();
            }
            if (this.element.is(':visible')) {
                that.resizeSlides();
                this.element.data('initState', 'complete');
                this.element.trigger('slideshow.init');
            } else {
                this.element.closest('.toc-content').on('toc.panelVisible', function(e) {
                    timeout = setTimeout(function(e) {
                        that.resizeSlides();
                        if (that.element.data('initState') != 'complete') {
                            that.element.data('initState', 'complete');
                            that.element.trigger('slideshow.init');
                        }
                    }, 0);
                });
            }

            $(window).on('resize', function(e) {
                clearTimeout(resizeTimer);
                resizeTimer = setTimeout(function() {
                    that.resizeSlides();
                }, 250);

            });
        },

        enableAutoScroll: function() {
            var that = this;
            var container = this.element.find(this.options.slide).closest('ul');

            if (this.slides.length <= 1) {
                return;
            }
            var timer = setInterval(function() {
                that.gotoSlide(that.index + 1);
            }, that.options.speed);

            container.on('mouseenter', function() {
                clearInterval(timer);
            });
            container.on('mouseleave', function() {
                timer = setInterval(function() {
                    that.gotoSlide(that.index + 1);
                }, that.options.speed);
            });
        },

        resizeSlides: function() {
            var that = this;
            var slideHeight = 0;

            that.width = this.element.width();
            if (this.options.setParentWidth === true) {
                this.slides.parent().width(that.width);
            }

            this.slides.each(function(i, slide) {
                $(slide).width(that.width);
                if (i !== that.index) {
                    $(slide).css('left', that.width);
                }
            });

            this.slides.each(function(i, slide) {
                slideHeight = $(slide).outerHeight() > slideHeight ? $(slide).outerHeight() : slideHeight;
            });

            this.element.find('.list').css('height', slideHeight);
            this.element.find('iframe').css('height', slideHeight - 5); // set video iframe height to slide height
        },

        enableTouchSupport: function() {
            var that = this;
            var startX = 0;
            var startY = 0;
            var endX = 0;
            var endY = 0;

            this.element[0].addEventListener('touchstart', function(e) {
                startX = e.touches[0].clientX;
                startY = e.touches[0].clientY;
                endX = startX;
                endY = startY;
            });

            this.element[0].addEventListener('touchmove', function(e) {
                endX = e.touches[0].clientX;
                endY = e.touches[0].clientY;
            });

            this.element[0].addEventListener('touchend', function(e) {
                if (Math.abs(startX - endX) > 50 && Math.abs(startY - endY) < 100) {
                    e.preventDefault();
                    that.gotoSlide(that.index + (startX > endX ? 1 : -1));
                }
            });
        },

        gotoSlide: function(i) {
            var direction;
            var $oldSlide;
            var $newSlide;
            var $iframe;
            var width;
            var src;
            var timeout;

            this.slides.find(this.options.slide).removeClass('active');

            if (i === this.index || this.slides.length === 0) {
                return;
            }

            direction = i > this.index ? 1 : -1;

            if (i < 0) {
                i = this.slides.length - 1;
            }

            if (i >= this.slides.length) {
                i = 0;
            }

            $oldSlide = $(this.slides[this.index]);

            // check if active youtube iframe in slide and fire 'slideHasIframe' event
            $iframe = $oldSlide.find('iframe[src]');
            if ($iframe.length) {
                $oldSlide.trigger('slideshow.slideHasIframe');
            }

            $newSlide = $(this.slides[i]);
            width = this.element.width();

            $newSlide.find('img').each(function(i, img) {
                if (img.src) {
                    return;
                }

                src = img.getAttribute('data-src');

                if (src) {
                    img.src = src;
                }
            });

            this.$caption.text($newSlide.attr('data-caption'));

            $oldSlide.removeClass('animate active');
            $newSlide.removeClass('animate').css('left', width * direction);

            // delay to not animate setting newSlide's inital left position
            timeout = setTimeout(function() {
                $oldSlide.addClass('animate').css('left', -width * direction);
                $newSlide.addClass('animate active').css('left', 0);
            });
            this.index = i;
            this.element.trigger('gotoSlide', i);
        },

        getNumSlides: function() {
            return this.slides.length;
        }
    });
})(jQuery);
