/**
 * @author till.reitemeyer + andy.gutsche
 */
(function($) {
    $.widget('carousel.slideshowThumbs', {
        _create: function() {
            var that = this;

            this.$slideshow = $(this.element.attr('data-slideshow'));

            if (this.$slideshow.length === 0) {
                return;
            }

            this.loadedThumbImgs = 0;
            this.$thumbImgs = $('img', this.element);
            this.totalThumbImgs = this.$thumbImgs.length;

            if (this.$slideshow.data('initState') !== 'complete') {
                $(document).one('slideshow.init', this.$slideshow, function() {
                    that._checkThumbLoadProgress();
                });
            } else {
                this._checkThumbLoadProgress();
            }

        },

        _checkThumbLoadProgress: function() {
            var timeout;
            var that = this;

            this.loadedThumbImgs = 0;

            this.$thumbImgs.each(function() {
                if (this.complete) {
                    that.loadedThumbImgs++;
                }
            });

            if (this.loadedThumbImgs === this.totalThumbImgs) {
                clearTimeout(timeout);
                this._initThumbSlider();
            } else {
                timeout = setTimeout(function(e) {
                    that._checkThumbLoadProgress();
                }, 250);
            }
        },

        _initThumbSlider: function() {
            var that = this;

            this.$slideshow = $(this.element.attr('data-slideshow'));

            if (this.$slideshow.length === 0) {
                return;
            }

            this.$prev = this.element.find('.prev');
            this.$next = this.element.find('.next');
            this.thumbs = this.element.find('.thumb');
            this.$skid = this.element.find('.skid');

            if (this.thumbs.length === 0) {
                this.element.hide();
                return;
            }

            this.index = 0;
            this.resetWidth();

            this.thumbs.on('click', function(e) {
                that.index = $(e.currentTarget).index();
                that.$slideshow.slideshow('gotoSlide', that.index);
            });

            this.$prev.on('click', function(e) {
                that.gotoThumb(that.index - that.numVisible);
            });

            this.$next.on('click', function(e) {
                that.gotoThumb(that.index + that.numVisible);
            });

            this.$slideshow.on('gotoSlide', function(e, i) {
                that.thumbs.removeClass('active');
                $(that.thumbs[i]).addClass('active');

                if (i < that.index || i >= that.index + that.numVisible) {
                    if (i > that.index) {
                        that.gotoThumb(i);
                    } else {
                        that.gotoThumb(i - (that.numVisible - 1));
                    }
                }

            });

            if (window.Modernizr && Modernizr.touch) {
                this.enableTouchSupport();
            }

            $(window).on('resize', function(e) {
                that.resetWidth();
            });
        },

        enableTouchSupport: function() {
            var that = this;
            var startX = 0;
            var startY = 0;
            var endX = 0;
            var endY = 0;
            var $thumbs = this.element.find('.thumbs')[0];

            $thumbs.addEventListener('touchstart', function(e) {
                startX = e.touches[0].clientX;
                startY = e.touches[0].clientY;
                endX = startX;
                endY = startY;
            });

            $thumbs.addEventListener('touchmove', function(e) {
                endX = e.touches[0].clientX;
                endY = e.touches[0].clientY;
            });

            $thumbs.addEventListener('touchend', function(e) {
                if (Math.abs(startX - endX) > 50 && Math.abs(startY - endY) < 100) {
                    e.preventDefault();
                    that.gotoThumb(that.index + (that.numVisible * (startX > endX ? 1 : -1)));
                }
            });
        },

        resetWidth: function() {
            var elPad;
            var maxWidth;
            var firstThumb = $(this.thumbs[0]);

            this.thumbWidth = firstThumb.outerWidth(true);
            this.element.css('max-width', 'none');
            this.numVisible = Math.floor(this.element.find('.thumbs').outerWidth() / this.thumbWidth);
            elPad = parseInt(this.element.css('padding-left'), 10) + parseInt(this.element.css('padding-right'), 10);

            if (this.thumbs.length <= this.numVisible) {
                maxWidth = this.thumbs.length * this.thumbWidth + this.$prev.outerWidth(true) + this.$next.outerWidth(true) + elPad;
                this.$prev.css('visibility', 'hidden');
                this.$next.css('visibility', 'hidden');
            } else {
                maxWidth = this.numVisible * this.thumbWidth + this.$prev.outerWidth(true) + this.$next.outerWidth(true) + elPad;
                this.$prev.css('visibility', 'visible');
                this.$next.css('visibility', 'visible');
            }

            this.element.css('max-width', maxWidth);
            this.gotoThumb(this.index);
        },

        gotoThumb: function(i) {
            var maxIndex;

            if (i < 0) {
                i = 0;
            }

            maxIndex = this.thumbs.length - this.numVisible;

            if (i >= maxIndex) {
                i = maxIndex;
            }

            if (maxIndex < 0) {
                i = 0;
            }

            this.$skid.css('left', -i * this.thumbWidth);
            this.index = i;
        }
    });
})(jQuery);
